-- Function: public.adicionar_quebra_linha()

-- DROP FUNCTION public.adicionar_quebra_linha();

CREATE OR REPLACE FUNCTION public.adicionar_quebra_linha()
  RETURNS character varying AS
$BODY$
DECLARE
    ehTxt BOOLEAN;
BEGIN
    -- Chama a funo anterior para verificar se o texto contm uma porta ou caminho vlido
    ehTxt := ImpressaoEhTxt();

    -- Se for um arquivo de texto, retorna a tag '</lf>', caso contrrio, retorna vazio ''
    RETURN CASE
        WHEN ehTxt THEN
            '</lf>'
        ELSE
            ''
    END;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION public.adicionar_quebra_linha()
  OWNER TO postgres;


-- Function: public.alinhar_texto(text[], text[])

-- DROP FUNCTION public.alinhar_texto(text[], text[]);

CREATE OR REPLACE FUNCTION public.alinhar_texto(
    textos text[],
    alinhamentos text[])
  RETURNS text AS
$BODY$
DECLARE
    resultado text := '';
    max_colunas integer := 48;
    i integer;
BEGIN
    FOR i IN 1..array_length(textos, 1) LOOP
        -- Verifica se o texto cabe na coluna
        IF length(resultado || textos[i]) <= max_colunas THEN
            resultado := resultado || textos[i];
        ELSE
            -- Se no couber, alinha conforme o alinhamento
            CASE alinhamentos[i]
                WHEN 'Esquerda' THEN
                    resultado := resultado || substring(textos[i] FROM 1 FOR max_colunas - length(resultado));
                WHEN 'Centro' THEN
                    -- Calcula o espao em branco necessrio
                    DECLARE
                        espaco integer := max_colunas - length(resultado) - length(textos[i]);
                        espaco_esquerda integer := espaco / 2;
                        espaco_direita integer := espaco - espaco_esquerda;
                    BEGIN
                        resultado := resultado || lpad('', espaco_esquerda, ' ') || textos[i] || rpad('', espaco_direita, ' ');
                    END;
                WHEN 'Direita' THEN
                    resultado := resultado || lpad('', max_colunas - length(resultado), ' ') || textos[i];
            END CASE;
        END IF;

        -- Adiciona espao entre as colunas
        IF i < array_length(textos, 1) THEN
            resultado := resultado || ' ';
        END IF;
    END LOOP;

    -- Preenche com espaos em branco at completar as 48 colunas
    IF length(resultado) < max_colunas THEN
        resultado := resultado || lpad('', max_colunas - length(resultado), ' ');
    END IF;

    RETURN resultado;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION public.alinhar_texto(text[], text[])
  OWNER TO postgres;


-- Function: public.alinhar_texto(character varying, character varying)

-- DROP FUNCTION public.alinhar_texto(character varying, character varying);

CREATE OR REPLACE FUNCTION public.alinhar_texto(
    texto character varying,
    tipo_alinhamento character varying)
  RETURNS character varying AS
$BODY$
DECLARE
    largura_maxima INT := 48;
    espacos_esquerda INT;
    espacos_direita INT;
    espacos_total INT;
BEGIN
    -- Calcula a quantidade de espaos a serem adicionados  esquerda e  direita
    espacos_total := largura_maxima - LENGTH(texto);
    
    IF espacos_total <= 0 THEN
        -- Se o texto for mais longo ou igual  largura mxima, retorna o texto original
        RETURN texto;
    END IF;

    CASE tipo_alinhamento
        WHEN 'esquerda' THEN
            RETURN texto || RPAD(' ', espacos_total, ' ');
        WHEN 'direita' THEN
            RETURN LPAD(' ', espacos_total, ' ') || texto;
        WHEN 'centro' THEN
            espacos_esquerda := FLOOR(espacos_total / 2);
            espacos_direita := CEIL(espacos_total / 2);
            RETURN LPAD(' ', espacos_esquerda, ' ') || texto || RPAD(' ', espacos_direita, ' ');
        ELSE
            -- Se o tipo de alinhamento no for reconhecido, retorna o texto original
            RETURN texto;
    END CASE;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION public.alinhar_texto(character varying, character varying)
  OWNER TO postgres;


-- Function: public.alinhar_texto_array(character varying[], character varying[])

-- DROP FUNCTION public.alinhar_texto_array(character varying[], character varying[]);

CREATE OR REPLACE FUNCTION public.alinhar_texto_array(
    textos character varying[],
    alinhamentos character varying[])
  RETURNS character varying AS
$BODY$
DECLARE
    largura_maxima INT := 48;
    texto_resultante VARCHAR := '';
    i INT;
    espacos_esquerda INT;
    espacos_direita INT;
    espacos_total INT;
BEGIN
    FOR i IN 1..array_length(textos, 1) LOOP
        -- Calcula a quantidade de espaos a serem adicionados  esquerda e  direita
        espacos_total := largura_maxima - LENGTH(texto_resultante);

        IF espacos_total <= 0 THEN
            -- Se o texto atual j atingiu a largura mxima, retorna o texto atual
            RETURN texto_resultante;
        END IF;

        CASE alinhamentos[i]
            WHEN 'esquerda' THEN
                texto_resultante := texto_resultante || textos[i] || repeat(' ', largura_maxima - LENGTH(texto_resultante));
            WHEN 'direita' THEN
                texto_resultante := texto_resultante || repeat(' ', espacos_total) || textos[i];
            WHEN 'centro' THEN
                espacos_esquerda := FLOOR(espacos_total / 2);
                espacos_direita := CEIL(espacos_total / 2);
                texto_resultante := texto_resultante || repeat(' ', espacos_esquerda) || textos[i] || repeat(' ', espacos_direita);
            ELSE
                -- Se o tipo de alinhamento no for reconhecido, retorna o texto atual
                RETURN texto_resultante;
        END CASE;
    END LOOP;

    -- Retorna o texto resultante aps o alinhamento
    RETURN texto_resultante;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION public.alinhar_texto_array(character varying[], character varying[])
  OWNER TO postgres;


-- Function: public.formatar_texto(character varying[], character[])

-- DROP FUNCTION public.formatar_texto(character varying[], character[]);

CREATE OR REPLACE FUNCTION public.formatar_texto(
    textos character varying[],
    alinhamentos character[])
  RETURNS character varying AS
$BODY$
DECLARE
    resultado VARCHAR := '';
    texto_atual VARCHAR;
    alinhamento_atual CHAR;
BEGIN
    FOR i IN 1..GREATEST(array_length(textos, 1), array_length(alinhamentos, 1)) LOOP
        texto_atual := COALESCE(textos[i], '');
        alinhamento_atual := COALESCE(alinhamentos[i], 'L');

        CASE alinhamento_atual
            WHEN 'L' THEN
                resultado := resultado || LPAD(SUBSTRING(texto_atual || RPAD('', 48) FROM 1 FOR 48), 48);
            WHEN 'R' THEN
                resultado := resultado || LPAD(SUBSTRING(LPAD('', 48) || texto_atual FROM 1 FOR 48), 48);
            WHEN 'C' THEN
                resultado := resultado || LPAD(SUBSTRING(LPAD('', FLOOR((48 - CHAR_LENGTH(texto_atual)) / 2)) || texto_atual || RPAD('', CEIL((48 - CHAR_LENGTH(texto_atual)) / 2)) FROM 1 FOR 48), 48);
        END CASE;

        resultado := resultado || ' ';
    END LOOP;

    RETURN TRIM(resultado);
END;
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION public.formatar_texto(character varying[], character[])
  OWNER TO postgres;


-- Function: public.formulasql(text, character varying, character varying)

-- DROP FUNCTION public.formulasql(text, character varying, character varying);

CREATE OR REPLACE FUNCTION public.formulasql(
    text,
    character varying,
    character varying)
  RETURNS text AS
$BODY$
DECLARE
StrSQL ALIAS FOR $1 ;
Param1 ALIAS FOR $2 ;
Param2 ALIAS FOR $3 ;
Comando Text;
Retorno VarChar;
Vazio Varchar;
 BEGIN
	Comando := StrSQL;
	Comando := Replace( Comando, '$1', Param1 );
	Comando := Replace( Comando, '$2', Param2 );
	EXECUTE Comando INTO Retorno;
	RETURN Retorno;
 EXCEPTION            
    WHEN others THEN RETURN 'ERRO: ' || SQLERRM ;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION public.formulasql(text, character varying, character varying)
  OWNER TO postgres;


-- Function: public.import_text_file(text)

-- DROP FUNCTION public.import_text_file(text);

CREATE OR REPLACE FUNCTION public.import_text_file(file_path text)
  RETURNS integer AS
$BODY$
DECLARE
    content TEXT;
BEGIN
    -- Construir o caminho completo relativo ao diretrio de dados do PostgreSQL
    -- Certifique-se de que o diretrio seja acessvel pelo PostgreSQL
    -- Exemplo: '/caminho/para/seu/diretorio/data/' || file_path

    -- Ler o contedo do arquivo de texto
    EXECUTE 'COPY tabela_exemplo (conteudo) FROM  ''' || file_path || ''' ';

    -- Retornar algum valor indicando sucesso (opcional)
    RETURN 1;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION public.import_text_file(text)
  OWNER TO postgres;


-- Function: public.importareorganizationscript(text)

-- DROP FUNCTION public.importareorganizationscript(text);

CREATE OR REPLACE FUNCTION public.importareorganizationscript(caminhoreorganizationscript text)
  RETURNS integer AS
$BODY$
DECLARE
    content TEXT;
BEGIN
    -- Ler o contedo do arquivo de texto
     EXECUTE 'COPY ReorganizationScript (Conteudo) FROM ''' || CaminhoReorganizationScript || '''';

    -- Exclui as linhas em branco do .txt	
    DELETE FROM ReorganizationScript
     WHERE Conteudo = ''
        OR Conteudo IS NULL;
       
    -- Atualizar os campos adicionais
    UPDATE ReorganizationScript
    SET
        Completo = False, 
        DataHora = Now()
    WHERE
        Completo IS NULL;

    -- Coloca ';' no fim da Consulta
    UPDATE ReorganizationScript
       SET Conteudo = Conteudo || ';'
     WHERE Completo = False
       AND Conteudo not like '%;%';
	
    -- Agrupar todos os inserts em um nico insert
    INSERT INTO ReorganizationScript (Conteudo, DataHora, Completo)
    SELECT
	    STRING_AGG(Conteudo, ' ') AS Conteudo,
	    NOW() AS DataHora,
	    TRUE AS Completo
	FROM (
	    SELECT Conteudo
	    FROM ReorganizationScript
	    WHERE Completo = FALSE AND Conteudo <> ''
	    ORDER BY Id
	) AS subquery;

    -- Deletar os registros desnecessrios
    DELETE FROM ReorganizationScript
    WHERE Completo = False;
    
    -- Retornar algum valor indicando sucesso (opcional)
    RETURN 1;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION public.importareorganizationscript(text)
  OWNER TO postgres;


-- Function: public.impressaoehtxt()

-- DROP FUNCTION public.impressaoehtxt();

CREATE OR REPLACE FUNCTION public.impressaoehtxt()
  RETURNS boolean AS
$BODY$
DECLARE
    ambiente_info RECORD;
BEGIN
    -- Obtm as informaes do Ambiente 
    SELECT * INTO ambiente_info FROM Ambiente;

    -- Verifica se AmbienteImpressoraPorta contm "COM3" ou "USB" ou um caminho no sistema
    RETURN CASE
        WHEN POSITION('/' IN ambiente_info.AmbienteImpressoraPorta) > 0 OR POSITION('.' IN ambiente_info.AmbienteImpressoraPorta) > 0 OR ambiente_info.AmbienteImpressoraPorta = '' THEN
            TRUE
        ELSE
            FALSE
    END;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION public.impressaoehtxt()
  OWNER TO postgres;


-- Function: public.postgres_fdw_handler()

-- DROP FUNCTION public.postgres_fdw_handler();

CREATE OR REPLACE FUNCTION public.postgres_fdw_handler()
  RETURNS fdw_handler AS
'$libdir/postgres_fdw', 'postgres_fdw_handler'
  LANGUAGE c VOLATILE STRICT
  COST 1;
ALTER FUNCTION public.postgres_fdw_handler()
  OWNER TO postgres;


-- Function: public.postgres_fdw_validator(text[], oid)

-- DROP FUNCTION public.postgres_fdw_validator(text[], oid);

CREATE OR REPLACE FUNCTION public.postgres_fdw_validator(
    text[],
    oid)
  RETURNS void AS
'$libdir/postgres_fdw', 'postgres_fdw_validator'
  LANGUAGE c VOLATILE STRICT
  COST 1;
ALTER FUNCTION public.postgres_fdw_validator(text[], oid)
  OWNER TO postgres;
